'use strict';
evaidyaApp.controller('BPListCtrl', ['$scope','$stateParams','$rootScope', '$location','$ionicPopup', 'BPData', function BPListCtrl($scope,$stateParams, $rootScope, $location, $ionicPopup, BPData) {
    $scope.viewOnly = false;
    $scope.chartDataSource = [];
    $scope.trendchartDataSource = [];
    $scope.itemButtons = [
        {
            text: 'Delete',
            type: 'button-assertive',
            onTap: function(item) {
                $scope.deleteItem(item);            }
        },
        {
            text: 'Edit',
            type: 'button-calm',
            onTap: function(item) {
                var hashpath = "/evaidya/bpdetail/"+ item;
                $location.path(hashpath);
            }
        }
    ];

    if($stateParams.id == 0)
        $scope.istrend = true;

    $scope.chartOptions = {
        startValue: 0,
        endValue: 150,
        values: $scope.chartDataSource,
        tooltip: { enabled: true },
        title: {
            text: 'Latest BP Reading',
            font: {
                size: 12,
                weight: 700
            },
            position: 'bottom-center'
        },
        label: {
            format: 'decimal',
            indent: 5,
            font: {
                size: 12,
                weight: 700
            },
            customizeText: function () {
                var bptype;
                switch (this.index) {
                    case 0: bptype = 'Systolic'
                        break;
                    case 1: bptype = 'Mean'
                        break;
                    case 2: bptype = 'Diastolic'
                        break;
                    default: bptype = undefined
                        break;
                }
                return bptype + ': ' + this.valueText;
            }
        },
        palette: 'ocean'
    };


    $scope.trendOptions = {
        dataSource: $scope.trendchartDataSource,
        commonSeriesSettings: {
            type: 'spline',
            argumentField: 'measureDate'
        },
        legend: {
            verticalAlignment: 'bottom',
            horizontalAlignment: 'center',
            itemTextPosition: 'right'
        },
        series: [
            { valueField: 'Systolic', name: 'Systolic' },
            { valueField: 'Mean', name: 'Mean' },
            { valueField: 'Diastolic', name: 'Diastolic' }
        ],
        margin: {
            left : 20,
            top: 20
        },
        valueAxis: {
            min: 0,
            max: 200,
            constantLines: [{
                label: {
                    text: '(D)-Low Normal'
                },
                width: 2,
                value: 60,
                color: '#ff4500',
                dashStyle: 'dash'
            }, {
                label: {
                    text: '(D)-High,(S)-Low Normals'
                },
                width: 2,
                value: 90,
                color: '#00ced1',
                dashStyle: 'dash'
            }, {
                label: {
                    text: '(S)-High Normal'
                },
                width: 2,
                value: 120,
                color: '#ff4500',
                dashStyle: 'dash'
            }]
        },
        tooltip: { enabled: true },

        palette: 'ocean'
    };

    if($scope.istrend)
    {
        $("#bpChart").dxBarGauge($scope.chartOptions);
        $("#trendChart").dxChart($scope.trendOptions);
    }

    $scope.updateChartData = function(){
        if($scope.BPList.length > 0)
        {
            $scope.trendchartDataSource = [];
            for (var i=$scope.BPList.length -1 ;i>=0;i--)
            {
                if(i===0) // show the first item
                    $scope.chartDataSource = [$scope.BPList[i].systolicBP, $scope.BPList[i].meanArterialBP,$scope.BPList[i].diastolicBP];
                $scope.trendchartDataSource.push({measureDate: $scope.BPList[i].measureDate.toLocaleDateString(), Systolic: $scope.BPList[i].systolicBP, Mean: $scope.BPList[i].meanArterialBP,Diastolic:$scope.BPList[i].diastolicBP});
            };
        }
/*        else // populate some sample data
        {
            $scope.chartDataSource = [80,  90.6, 120];
            $scope.trendchartDataSource = [
                {
                    measureDate: "1/1/2001", Systolic: 80, Mean: 90,  Diastolic: 120
                },
                {
                    measureDate: "1/1/2005", Systolic: 85, Mean: 95,  Diastolic: 130
                },
                {
                    measureDate: "1/1/2010", Systolic: 87, Mean: 99,  Diastolic: 135
                }
            ];
        }
*/
    }

    if(BPData.uname === $scope.uname && BPData.BPList )
    {

        $scope.BPList =  BPData.BPList;
        $scope.plotseries = BPData.plotseries;
        if($scope.istrend)
        {
            $scope.updateChartData();
            $("#trendChart").dxChart('option','dataSource',$scope.trendchartDataSource);
            $("#bpChart").dxBarGauge('option','values',$scope.chartDataSource);
        }
    }
    else
    {
        BPData.getBPList().then(function (BPList) {
            $scope.BPList = BPList;
            BPData.BPList = BPList;
            BPData.uname = $scope.uname;
            if($scope.istrend)
            {
                $scope.updateChartData();
                $("#trendChart").dxChart('option','dataSource',$scope.trendchartDataSource);
                $("#bpChart").dxBarGauge('option','values',$scope.chartDataSource);
            }
        }, function (response) {
            console.log(response);
        });
    }


    $scope.profileusername="Welcome <b>"+profileusername+"</b>";


    $scope.deleteItem = function (listIndex) {
        $ionicPopup.confirm({
            title: 'Delete BP Record?',
            content: 'Are you sure you want to delete?'
        }).then(function(res) {
                if(res) {
            $scope.BPList = BPData.BPList;
            BPData.BP.delete({uname: $scope.uname, id: $scope.BPList[listIndex]._id}, function () {
                    if ($scope.BPList)
                        $scope.BPList.splice(listIndex, 1);
                        },
                        function (response) {
                        });
                } else {
                }
            });
     };
}]);

evaidyaApp.controller('BPDetailCtrl',['$scope', '$location', '$stateParams','$filter','BPData', function BPDetailCtrl($scope, $location, $stateParams,$filter,BPData) {
    var isNew = false;

    BPData.getBPList().then(function (BPList) {
        $scope.BPList = BPList;
        BPData.BPList = BPList;
        var routeid = Number($stateParams.id);
        if(routeid >= 0)
        // shallow copy the object
            $scope.BPData = jQuery.extend({}, BPData.BPList[routeid]);
        else if($stateParams.id === "-1")
        {
            $scope.BPData = {measureDate:new Date()
                , systolicBP:0
                , diastolicBP:0
                , meanArterialBP: 0
                ,patientId: $scope.getUserName()};
            isNew = true;
        }
        else // clone the first element
        {
            $scope.BPData = jQuery.extend({}, $scope.BPList[0]);
            isNew = true;
        }
        $scope.BPData.measureDate = $filter("date")($scope.BPData.measureDate, 'yyyy-MM-dd');
    });


    $scope.save  = function () {
        if(isNew)
        {
            BPData.bp.save({uname:$scope.uname},$scope.BPData,function (event) {
                    BPData.BPList.push(event);
                    event.measureDate = new Date(event.measureDate);
                    // sort - very important
                    BPData.BPList.sort(function (el1, el2) {
                        return (el2.measureDate > el1.measureDate) ? 1 : ( (el1.measureDate > el2.measureDate) ? -1 : 0 );
                    });
                    $scope.$back();
                },
                function (response) {});
        }
        else
            BPData.bp.update({uname:$scope.uname},$scope.BPData,function (event) {
                    event.measureDate = new Date(event.measureDate);
                    jQuery.each(BPData.BPList, function(j, el) {
                        if (event._id === el._id)
                        {
                            BPData.BPList[j] = event;
                            return false; // break the loop
                        }
                    });
                    // sort - very important
                    BPData.BPList.sort(function (el1, el2) {
                        return (el2.measureDate > el1.measureDate) ? 1 : ( (el1.measureDate > el2.measureDate) ? -1 : 0 );
                    });
                    $scope.$back();
                },
                function (response) {});

    };

}]);